#include <algorithm>
#include <array>
#include <cassert>
#include <climits>
#include <cmath>
#include <iomanip>
#include <iostream>
#include <map>
#include <random>
#include <unordered_map>
#include <vector>

#include "circle.h"
// #include "../contestant/Lgrader.cpp"

struct Point {
    long double x, y;
};

struct Circle {
    long double x, y, r;

    bool operator<(const Circle& other) const {
        if(x != other.x) {
            return x < other.x;
        }
        if(y != other.y) {
            return y < other.y;
        }
        return r < other.r;
    }
};

long double cross(const Point& a, const Point& b) {
    return a.x * b.y - a.y * b.x;
}

Point vector_between(const Point& a, const Point& b) {
    return Point{b.x - a.x, b.y - a.y};
}

Point midpoint(const Point& a, const Point& b) {
    return Point{(a.x + b.x) / 2, (a.y + b.y) / 2};
}

Point perpendicular(const Point& v) { return Point{-v.y, v.x}; }

long double distance(const Point& a, const Point& b) {
    long double dx = b.x - a.x;
    long double dy = b.y - a.y;
    return std::sqrt(dx * dx + dy * dy);
}

Circle get_circle_from_3_points(
    const Point& A, const Point& B, const Point& C
) {
    Point v1 = vector_between(A, B);
    Point v2 = vector_between(A, C);
    if(std::abs(cross(v1, v2)) < 1e-10) {
        return Circle{0, 0, -1};
    }

    Point M1 = midpoint(A, B);
    Point M2 = midpoint(B, C);

    Point V1 = perpendicular(vector_between(A, B));
    Point V2 = perpendicular(vector_between(B, C));

    long double cross_v = cross(V1, V2);
    if(std::abs(cross_v) < 1e-10) {
        return Circle{0, 0, -1};
    }

    Point M2_M1 = vector_between(M1, M2);
    long double t = cross(M2_M1, V2) / cross_v;

    Point O{M1.x + t * V1.x, M1.y + t * V1.y};
    long double r = distance(O, A);

    return Circle{O.x, O.y, r};
}

bool is_on_circle(const Point& p, const Circle& c) {
    long double d = distance(Point{c.x, c.y}, p);
    return std::abs(d - c.r) < 1e-6;
}

void solve(int k) {
    std::vector<Point> points;
    std::vector<Circle> confirmed_circles;
    std::map<std::tuple<int64_t, int64_t, int64_t>, int> circle_counts;
    std::map<std::tuple<int64_t, int64_t, int64_t>, Circle> real_circles;
    const long double eps = 1e-5;

    while(confirmed_circles.size() < k) {
        // Sample a new point
        auto [x, y] = sample_star();
        Point new_point{x, y};

        // Check if the new point lies on any confirmed circle
        bool on_confirmed_circle = false;
        for(const auto& circle: confirmed_circles) {
            if(is_on_circle(new_point, circle)) {
                on_confirmed_circle = true;
                break;
            }
        }

        if(on_confirmed_circle) {
            continue;
        }
        
        // Try forming circles with all pairs of existing points
        for(int i = 0; i < points.size(); i++) {
            for(int j = i + 1; j < points.size(); j++) {
                Circle circle =
                    get_circle_from_3_points(points[i], points[j], new_point);
                if(circle.r <= 0) {
                    continue;
                }

                // Check for overflow
                if(std::abs(circle.x / eps) > (long double)INT64_MAX ||
                   std::abs(circle.y / eps) > (long double)INT64_MAX ||
                   std::abs(circle.r / eps) > (long double)INT64_MAX) {
                    continue;
                }        

                int64_t base_x = (int64_t)(circle.x / eps + 0.5);
                int64_t base_y = (int64_t)(circle.y / eps + 0.5);
                int64_t base_r = (int64_t)(circle.r / eps + 0.5);

                auto key = std::make_tuple(base_x, base_y, base_r);
                circle_counts[key]++;
                real_circles[key] = circle;

                // If circle count exceeds threshold, confirm it
                bool is_new_circle = true;
                for (const auto& existing : confirmed_circles) {
                    if (std::abs(existing.x - circle.x) < 1e-4 && 
                        std::abs(existing.y - circle.y) < 1e-4 && 
                        std::abs(existing.r - circle.r) < 1e-4) {
                        is_new_circle = false;
                        break;
                    }
                }

                if (circle_counts[key] > 56 && is_new_circle) {
                    confirmed_circles.push_back(circle);

                    // Remove all points that lie on this circle
                    std::vector<Point> new_points;
                    for (const auto& p : points) {
                        if (!is_on_circle(p, circle)) {
                            new_points.push_back(p);
                        }
                    }
                    points = new_points;
                }
            }
        }

        points.push_back(new_point);
    }

    // Output confirmed circles
    for(const auto& circle: confirmed_circles) {
        answer(circle.x, circle.y, circle.r);
    }
}
