#include <algorithm>
#include <array>
#include <cassert>
#include <climits>
#include <cmath>
#include <iomanip>
#include <iostream>
#include <map>
#include <random>
#include <unordered_map>
#include <vector>

#include "circle.h"

struct Point {
    long double x, y;
};

struct Circle {
    long double x, y, r;

    bool operator<(const Circle& other) const {
        if(x != other.x) {
            return x < other.x;
        }
        if(y != other.y) {
            return y < other.y;
        }
        return r < other.r;
    }
};

long double cross(const Point& a, const Point& b) {
    return a.x * b.y - a.y * b.x;
}

Point vector_between(const Point& a, const Point& b) {
    return Point{b.x - a.x, b.y - a.y};
}

Point midpoint(const Point& a, const Point& b) {
    return Point{(a.x + b.x) / 2, (a.y + b.y) / 2};
}

Point perpendicular(const Point& v) { return Point{-v.y, v.x}; }

long double distance(const Point& a, const Point& b) {
    long double dx = b.x - a.x;
    long double dy = b.y - a.y;
    return std::sqrt(dx * dx + dy * dy);
}

Circle get_circle_from_3_points(
    const Point& A, const Point& B, const Point& C
) {
    Point v1 = vector_between(A, B);
    Point v2 = vector_between(A, C);
    if(std::abs(cross(v1, v2)) < 1e-10) {
        return Circle{0, 0, -1};
    }

    Point M1 = midpoint(A, B);
    Point M2 = midpoint(B, C);

    Point V1 = perpendicular(vector_between(A, B));
    Point V2 = perpendicular(vector_between(B, C));

    long double cross_v = cross(V1, V2);
    if(std::abs(cross_v) < 1e-10) {
        return Circle{0, 0, -1};
    }

    Point M2_M1 = vector_between(M1, M2);
    long double t = cross(M2_M1, V2) / cross_v;

    Point O{M1.x + t * V1.x, M1.y + t * V1.y};
    long double r = distance(O, A);

    return Circle{O.x, O.y, r};
}

Point sample_point() {
    auto [x, y] = sample_star();
    return Point{x, y};
}

void solve(int k) {
    std::vector<Point> points;
    for(int i = 0; i < 10000; i++) {
        points.push_back(sample_point());
    }

    std::random_device rd;
    std::mt19937 gen(rd());
    std::uniform_int_distribution<> dis(0, points.size() - 1);

    std::map<std::tuple<int64_t, int64_t, int64_t>, int> circle_counts;
    std::map<std::tuple<int64_t, int64_t, int64_t>, Circle> real_circles;

    const int num_iterations = 300000;
    const long double eps = 1e-5;
    for(int i = 0; i < num_iterations; i++) {
        Point A = points[dis(gen)];
        Point B = points[dis(gen)];
        Point C = points[dis(gen)];

        Circle circle = get_circle_from_3_points(A, B, C);
        if(circle.r > 0) {
            // Check if values would overflow int64_t
            if(std::abs(circle.x / eps) > (long double)INT64_MAX ||
               std::abs(circle.y / eps) > (long double)INT64_MAX ||
               std::abs(circle.r / eps) > (long double)INT64_MAX) {
                continue;
            }

            int64_t base_x = (int64_t)(circle.x / eps + 0.5);
            int64_t base_y = (int64_t)(circle.y / eps + 0.5);
            int64_t base_r = (int64_t)(circle.r / eps + 0.5);

            circle_counts[{base_x, base_y, base_r}]++;
            real_circles[{base_x, base_y, base_r}] = circle;
        }
    }

    std::vector<std::pair<Circle, int>> circles;

    for(const auto& [circle, count]: circle_counts) {
        auto [x, y, r] = circle;
        circles.push_back({real_circles[{x, y, r}], count});
    }

    std::sort(circles.begin(), circles.end(), [](const auto& a, const auto& b) {
        return a.second > b.second;
    });

    assert(circles.size() >= k);
    circles.erase(circles.begin() + k, circles.end());

    for(const auto& [circle, count]: circles) {
        answer(circle.x, circle.y, circle.r);
    }
}
