#include <algorithm>
#include <cmath>
#include <fstream>
#include <iomanip>
#include <iostream>
#include <vector>

struct Circle {
    double x, y, r;
    Circle(double x_, double y_, double r_) : x(x_), y(y_), r(r_) {}
    bool operator<(const Circle& other) const {
        if(x != other.x) {
            return x < other.x;
        }
        if(y != other.y) {
            return y < other.y;
        }
        return r < other.r;
    }
};

int main(int argc, char* argv[]) {
    if(argc != 4) {
        std::cerr << "Usage: " << argv[0]
                  << " input_file jury_answer contestant_output\n";
        return 1;
    }

    std::ifstream input(argv[1]);
    std::ifstream jury(argv[2]);
    std::ifstream contestant(argv[3]);

    if(!input || !jury || !contestant) {
        std::cerr << "Error opening files\n";
        return 1;
    }

    // Read and compute hash of input
    const int mod = 999142949;
    int computed_hsh = 0;

    int N, K;
    input >> N >> K;

    std::vector<std::pair<double, double>> points(N);
    for(int i = 0; i < N; i++) {
        input >> points[i].first >> points[i].second;
        computed_hsh =
            (1LL * computed_hsh * (N + 43) + (long long)points[i].second) % mod;
        computed_hsh =
            (1LL * computed_hsh * (N + 53) + (long long)points[i].first) % mod;
    }

    int status;
    contestant >> status;

    int contestant_hsh, queries;
    contestant >> contestant_hsh;
    contestant >> queries;

    // Read circles from jury and contestant
    std::vector<Circle> jury_circles;
    std::vector<Circle> contestant_circles;

    double x, y, r;
    while(jury >> x >> y >> r) {
        jury_circles.emplace_back(x, y, r);
    }

    while(contestant >> x >> y >> r) {
        contestant_circles.emplace_back(x, y, r);
    }

    if(status == -1) {
        std::cerr << "Invalid number of answer() calls.\n";
        std::cout << "0.0\n";
        return 0;
    }

    if(status == -2) {
        std::cerr << "Invalid circle parameters.\n";
        std::cout << "0.0\n";
        return 0;
    }

    if(status == -3) {
        std::cerr << "Too many sample_star() queries.\n";
        std::cout << "0.0\n";
        return 0;
    }

    if(jury_circles.size() != contestant_circles.size()) {
        std::cerr << "Wrong number of circles\n";
        std::cout << "0.0\n";
        return 0;
    }

    if(contestant_hsh != computed_hsh) {
        std::cerr << "Invalid output - do not print to stdout.\n";
        std::cout << "0.0\n";
        return 0;
    }

    // Sort both sets of circles
    std::sort(jury_circles.begin(), jury_circles.end());
    std::sort(contestant_circles.begin(), contestant_circles.end());

    // Calculate total error
    long double total_error = 0;
    for(size_t i = 0; i < (int)jury_circles.size(); i++) {
        if(std::abs(jury_circles[i].x - contestant_circles[i].x) > 1e-3 ||
           std::abs(jury_circles[i].y - contestant_circles[i].y) > 1e-3 ||
           std::abs(jury_circles[i].r - contestant_circles[i].r) > 1e-3) {
            std::cerr << "Circles don't match.\n";
            std::cout << "0.0\n";
            return 0;
        }
    }

    double score = 0.3 + 0.7 * pow(500.0 / std::max(queries, 500), 0.5);
    std::cout << std::fixed << std::setprecision(10) << score << "\n";
    return 0;
}
