import argparse
from collections import defaultdict
from dataclasses import dataclass
from enum import Enum
import numpy as np
import os
from pathlib import Path
import random
from typing import List, Tuple


class Strategy(Enum):
    LINE = "line"
    RECTANGLE = "rectangle"
    RANDOM_INTERSECT = "random_intersect"
    RANDOM_DENSE = "random_dense"
    RANDOM_NO_INTERSECT = "random_no_intersect"


@dataclass
class Circle:
    x: float  # center x coordinate
    y: float  # center y coordinate
    r: float  # radius

    def intersects(self, other: "Circle") -> bool:
        dist = np.sqrt((self.x - other.x) ** 2 + (self.y - other.y) ** 2)
        return dist < (self.r + other.r)

    def contains(self, x: float, y: float) -> bool:
        dist = np.sqrt((self.x - x) ** 2 + (self.y - y) ** 2)
        return dist <= self.r

    def random_point(self) -> Tuple[float, float]:
        """Generate a random point uniformly on the circle's circumference"""
        angle = random.uniform(0, 2 * np.pi)
        return (
            self.x + self.r * np.cos(angle),
            self.y + self.r * np.sin(angle),
        )


class CircleTestGenerator:
    def __init__(self, N: int, K: int, max_coord: float, strategy: Strategy):
        self.N = N
        self.K = K
        self.max_coord = max_coord
        self.strategy = strategy
        self.circles: List[Circle] = []

    def generate_circles(self) -> List[Circle]:
        if self.strategy == Strategy.LINE:
            return self._generate_line_circles()
        elif self.strategy == Strategy.RECTANGLE:
            return self._generate_rectangle_circles()
        elif self.strategy == Strategy.RANDOM_INTERSECT:
            return self._generate_random_intersect_circles()
        elif self.strategy == Strategy.RANDOM_DENSE:
            return self._generate_dense_circles()
        else:  # RANDOM_NO_INTERSECT
            return self._generate_no_intersect_circles()

    def _generate_line_circles(self) -> List[Circle]:
        circles = []
        # Generate random base radius between 1 and max_coord/10
        base_radius = random.uniform(1, self.max_coord / 10)

        # Start from the leftmost position that ensures all circles fit
        total_length = (self.K - 1) * 2 * base_radius
        start_x = (self.max_coord - total_length) / 2

        for i in range(self.K):
            x = start_x + i * 2 * base_radius
            # Add some variation to radius while keeping circles non-overlapping
            radius = base_radius * random.uniform(0.5, 1.0)
            circles.append(Circle(x, self.max_coord / 2, radius))

        return circles

    def _generate_rectangle_circles(self) -> List[Circle]:
        circles = []
        min_radius = self.max_coord / (4 * np.sqrt(self.K))
        max_radius = self.max_coord / (2 * np.sqrt(self.K))

        for _ in range(self.K):
            radius = random.uniform(min_radius, max_radius)
            x = random.uniform(radius, self.max_coord - radius)
            y = random.uniform(radius, self.max_coord - radius)
            circles.append(Circle(x, y, radius))

        return circles

    def _generate_random_intersect_circles(self) -> List[Circle]:
        circles = []
        min_radius = self.max_coord / (4 * np.sqrt(self.K))
        max_radius = self.max_coord / (2 * np.sqrt(self.K))

        for _ in range(self.K):
            radius = random.uniform(min_radius, max_radius)
            x = random.uniform(0, self.max_coord)
            y = random.uniform(0, self.max_coord)
            circles.append(Circle(x, y, radius))

        return circles

    def _generate_dense_circles(self) -> List[Circle]:
        circles = []
        # Use smaller radii to pack circles more densely
        min_radius = self.max_coord / (6 * np.sqrt(self.K))
        max_radius = self.max_coord / (4 * np.sqrt(self.K))

        for _ in range(self.K):
            radius = random.uniform(min_radius, max_radius)
            x = random.uniform(radius, self.max_coord - radius)
            y = random.uniform(radius, self.max_coord - radius)
            circles.append(Circle(x, y, radius))

        return circles

    def _generate_no_intersect_circles(self) -> List[Circle]:
        circles = []
        attempts = 100  # Maximum attempts to place a circle
        min_radius = self.max_coord / (6 * np.sqrt(self.K))
        max_radius = self.max_coord / (4 * np.sqrt(self.K))

        for _ in range(self.K):
            circle = None
            for _ in range(attempts):
                radius = random.uniform(min_radius, max_radius)
                x = random.uniform(radius, self.max_coord - radius)
                y = random.uniform(radius, self.max_coord - radius)
                new_circle = Circle(x, y, radius)

                # Check if the new circle intersects with any existing circles
                if not any(new_circle.intersects(c) for c in circles):
                    circle = new_circle
                    break

            if circle is None:
                # If we couldn't place a non-intersecting circle, reduce the radius and try again
                radius = min_radius
                x = random.uniform(radius, self.max_coord - radius)
                y = random.uniform(radius, self.max_coord - radius)
                circle = Circle(x, y, radius)

            circles.append(circle)

        return circles

    def generate_points(self) -> Tuple[List[Tuple[float, float]], List[int]]:
        """Generate N points and their circle assignments"""
        self.circles = self.generate_circles()
        points = []
        assignments = []

        for _ in range(self.N):
            # Choose a random circle
            circle_idx = random.randrange(self.K)
            circle = self.circles[circle_idx]

            # Generate a random point in the circle
            point = circle.random_point()
            points.append(point)
            assignments.append(circle_idx)

        return points, assignments


# Example usage:
def generate_test_case(
    N: int, K: int, max_coord: float, strategy: Strategy
) -> Tuple[List[Tuple[float, float]], List[int], List[Circle]]:
    generator = CircleTestGenerator(N, K, max_coord, strategy)
    points, assignments = generator.generate_points()
    return points, assignments, generator.circles


def get_min_center_distance(circles: List[Circle]) -> float:
    """Calculate minimum distance between any two circle centers"""
    min_dist = float("inf")
    for i in range(len(circles)):
        for j in range(i + 1, len(circles)):
            dist = np.sqrt(
                (circles[i].x - circles[j].x) ** 2
                + (circles[i].y - circles[j].y) ** 2
            )
            min_dist = min(min_dist, dist)
    return min_dist


def write_test_case(
    base_path: Path, points: List[Tuple[float, float]], circles: List[Circle]
):
    # Calculate min/max coordinates
    min_x = min(x for x, _ in points)
    max_x = max(x for x, _ in points)
    min_y = min(y for _, y in points)
    max_y = max(y for _, y in points)
    min_center_dist = get_min_center_distance(circles)
    print(f"Test {base_path.stem} coordinate ranges:")
    print(f"  X: [{min_x:.2f}, {max_x:.2f}]")
    print(f"  Y: [{min_y:.2f}, {max_y:.2f}]")
    print(f"  Min center distance: {min_center_dist:.2f}")

    # Write input file (.in)
    in_path = Path(str(base_path) + ".in")
    with in_path.open("w") as f:
        f.write(f"{len(points)} {len(circles)}\n")
        for x, y in points:
            f.write(f"{x:.6f} {y:.6f}\n")

    # Write output file (.out)
    out_path = Path(str(base_path) + ".out")
    with out_path.open("w") as f:
        for circle in circles:
            f.write(f"{circle.x:.6f} {circle.y:.6f} {circle.r:.6f}\n")


def main():
    parser = argparse.ArgumentParser(description="Generate circle test cases")
    parser.add_argument(
        "--out_dir",
        type=str,
        default=os.path.join(os.getcwd(), "tests"),
        help="Output directory for test cases (default: ./tests)",
    )
    args = parser.parse_args()

    out_dir = Path(args.out_dir)
    out_dir.mkdir(parents=True, exist_ok=True)

    max_coord = 100000.0
    N = 100020

    groups = [
        (2, "Group1", 2, Strategy.RANDOM_NO_INTERSECT),
        (2, "Group2", 2, Strategy.LINE),
        (3, "Group2", 2, Strategy.RANDOM_INTERSECT),
        (3, "Group2", 2, Strategy.RANDOM_NO_INTERSECT),
        (3, "Group2", 2, Strategy.RANDOM_DENSE),
        (2, "Group3", 5, Strategy.LINE),
        (3, "Group3", 5, Strategy.RANDOM_INTERSECT),
        (3, "Group3", 5, Strategy.RANDOM_NO_INTERSECT),
        (3, "Group3", 5, Strategy.RANDOM_DENSE),
        (2, "Group4", 10, Strategy.LINE),
        (2, "Group4", 10, Strategy.RECTANGLE),
        (3, "Group4", 10, Strategy.RANDOM_INTERSECT),
        (3, "Group4", 10, Strategy.RANDOM_NO_INTERSECT),
        (3, "Group4", 10, Strategy.RANDOM_DENSE),
        (2, "Group5", None, Strategy.LINE),
        (2, "Group5", None, Strategy.RECTANGLE),
        (3, "Group5", None, Strategy.RANDOM_INTERSECT),
        (3, "Group5", None, Strategy.RANDOM_NO_INTERSECT),
        (3, "Group5", None, Strategy.RANDOM_DENSE),
    ]

    id_per_group = defaultdict(int)
    overall_max_x = float("-inf")
    overall_max_y = float("-inf")
    overall_min_x = float("inf")
    overall_min_y = float("inf")

    overall_test_id = 1
    for num_tests, group_name, K, strategy in groups:
        print(f"Generating {group_name}... {overall_test_id}")
        for _ in range(num_tests):
            if K is None:
                K = random.randint(15, 20)

            points, _, circles = generate_test_case(N, K, max_coord, strategy)
            id_per_group[group_name] += 1

            # Update overall min/max coordinates
            min_x = min(x for x, _ in points)
            max_x = max(x for x, _ in points)
            min_y = min(y for _, y in points)
            max_y = max(y for _, y in points)

            overall_min_x = min(overall_min_x, min_x)
            overall_max_x = max(overall_max_x, max_x)
            overall_min_y = min(overall_min_y, min_y)
            overall_max_y = max(overall_max_y, max_y)

            # Ensure points are random
            random.shuffle(points)
            write_test_case(
                out_dir / f"circle.{overall_test_id:02d}", points, circles
            )
            overall_test_id = overall_test_id + 1

    print("All test cases generated successfully.")
    print("\nOverall coordinate ranges across all tests:")
    print(f"  X: [{overall_min_x:.2f}, {overall_max_x:.2f}]")
    print(f"  Y: [{overall_min_y:.2f}, {overall_max_y:.2f}]")


if __name__ == "__main__":
    main()
